/**
 * @file fbt_syscalls_impl.h
 * This file implements type definitions for system calls for i386 and X86_64.
 *
 * Copyright (c) 2011 ETH Zurich
 *
 * @author Mathias Payer <mathias.payer@nebelwelt.net>
 *
 * $Date: 2011-03-18 19:09:01 +0100 (Fri, 18 Mar 2011) $
 * $LastChangedDate: 2011-03-18 19:09:01 +0100 (Fri, 18 Mar 2011) $
 * $LastChangedBy: payerm $
 * $Revision: 428 $
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 */
#ifndef FBT_SYSCALLS_H
#define FBT_SYSCALLS_H

#if defined(__x86_64__)
/* 64b syscall numbers needed here */
#define SYS_exit     60
#define SYS_write     1
#define SYS_open      2
#define SYS_close     3
#define SYS_lseek     8
#define SYS_mmap      9
#define SYS_munmap   11
#define SYS_mprotect 10

#define __syscall_clobber "r11","rcx","memory"
#define _syscall1(name,arg1,__res)                                      \
    __asm__ volatile ("syscall;"                                        \
                      : "=a"(__res)                                     \
                      : "0"(SYS_##name),"D"((long)(arg1))               \
                      : __syscall_clobber );

#define _syscall2(name,arg1,arg2,__res)                                 \
    __asm__ volatile ("syscall;"                                        \
                      : "=a"(__res)                                     \
                      : "0"(SYS_##name),"D"((long)(arg1)),              \
                        "S"((long)(arg2))                               \
                      : __syscall_clobber );

#define _syscall3(name,arg1,arg2,arg3,__res)                            \
    __asm__ volatile ("syscall;"                                        \
                      : "=a"(__res)                                     \
                      : "0"(SYS_##name), "D"((long)(arg1)),             \
                        "S"((long)(arg2)), "d"((long)(arg3))            \
                      : __syscall_clobber );

#define _syscall6(name,arg1,arg2,arg3,arg4,arg5,arg6,__res)             \
    __asm__ volatile("movq %5,%%r10 ;"                                  \
                     "movq %6,%%r8 ;"                                   \
                     "movq %7,%%r9 ;"                                   \
                     "syscall ;"                                        \
                     : "=a"(__res)                                      \
                     : "0"(SYS_##name), "D"((long)(arg1)),              \
                        "S"((long)(arg2)), "d"((long)(arg3)),           \
                        "g"((long)(arg4)), "g"((long)(arg5)),           \
                        "g"((long)(arg6))                               \
                      : __syscall_clobber );

#else  /* i386+ mode */

/* 32b syscall numbers needed here */
#define I386_NR_SYSCALLS 332  /* total # of system calls */

#define SYS_exit           1  /* ensure that we close BT */
#define SYS_write          4
#define SYS_open           5
#define SYS_close          6
#define SYS_creat          8
#define SYS_execve        11  /* redirected to auth_execve */
#define SYS_lseek         19
#define SYS_ptrace        26  /* deny */
#define SYS_signal        48  /* installs a new signal handler (deprecated) */
#define SYS_sigaction     67  /* installs a new signal handler */
#define SYS_mmap          90  /* redirected to auth_mmap */
#define SYS_munmap        91  /* TODO */
#define SYS_fstat        108  /* use new fstat syscall */
#define SYS_sigreturn    119  /* we should never see this syscall */
#define SYS_clone        120  /* initializes a new thread */
#define SYS_mprotect     125  /* redirect to auth_mprotect */
#define SYS_rt_sigreturn 173  /* we should never see this syscall */
#define SYS_rt_sigaction 174  /* install a new signal handler */
#define SYS_mmap2        192  /* redirected to auth_mmap */
#define SYS_unused1      222  /* deny */
#define SYS_unused2      223  /* deny */
#define SYS_unused3      251  /* deny */
#define SYS_exit_group   252  /* ensure that we close BT */
#define SYS_sys_setaltroot 285  /* deny */
#define SYS_openat       295

/* fast or slow system call? */
#if defined(__i686__)
#define ENTER_KERNEL "call *%%gs:0x10 ;"
#else
#define ENTER_KERNEL "int $0x80 ;"
#endif  /* __i686__ */

#define _syscall1(name,arg1,__res)                                      \
    __asm__ volatile ("push %%ebx ;"                                    \
                      "movl %2,%%ebx ; "                                \
                      ENTER_KERNEL                                      \
                      "pop %%ebx"                                       \
                      : "=a"(__res)                                     \
                      : "0"(SYS_##name), "ri"((long)(arg1))             \
                      : "memory");


#define _syscall2(name,arg1,arg2,__res)                                 \
    __asm__ volatile ("push %%ebx ; "                                   \
                      "movl %2,%%ebx ; "                                \
                      ENTER_KERNEL                                      \
                      "pop %%ebx"                                       \
                      : "=a"(__res)                                     \
                      : "0"(SYS_##name), "ri"((long)(arg1)),            \
                        "c"((long)(arg2))                               \
                      : "memory");

#define _syscall3(name,arg1,arg2,arg3,__res)                            \
    __asm__ volatile ("push %%ebx ;"                                    \
                      "movl %2,%%ebx ;"                                 \
                      ENTER_KERNEL                                      \
                      "pop %%ebx"                                       \
                      : "=a"(__res)                                     \
                      : "0"(SYS_##name), "ri"((long)(arg1)),            \
                        "c"((long)(arg2)), "d"((long)(arg3))            \
                      : "memory");

#define _syscall5(name,arg1,arg2,arg3,arg4,arg5,__res)                  \
    __asm__ volatile ("pushl %%ebx ;"                                   \
                      "movl %2,%%ebx ;"                                 \
                      "movl %1,%%eax ;"                                 \
                      ENTER_KERNEL                                      \
                      "pop %%ebx"                                       \
                      : "=a"(__res)                                     \
                      : "i"(SYS_##name), "ri"((long)(arg1)),           \
                        "c"((long)(arg2)), "d"((long)(arg3)),           \
                        "S"((long)(arg4)), "D"((long)(arg5))            \
                        : "memory");

/* #define _syscall5(name,arg1,arg2,arg3,arg4,arg5,__res) \ */
/*   __asm__ volatile ("int $0x80" \ */
/*   : "=a"(__res) \ */
/*   : "0"(SYS_##name), "b" ((long)(arg1)),"c" ((long)(arg2)), \ */
/*     "d" ((long)(arg3)),"S" ((long)(arg4)),"D" ((long)(arg5))) */

#define _syscall6(name,arg1,arg2,arg3,arg4,arg5,arg6,__res)             \
  __asm__ volatile("pushl %%ebx ;"                                      \
                   "pushl %2 ;"                                         \
                   "pushl %3 ;"                                         \
                   "pushl %4 ;"                                         \
                   "pushl %5 ;"                                         \
                   "pushl %6 ;"                                         \
                   "pushl %7 ;"                                         \
                   "movl %%esp, %%ebx ;"                                \
                   ENTER_KERNEL                                         \
                   "addl $24, %%esp ;"                                  \
                   "popl %%ebx"                                         \
                   : "=a"(__res)                                        \
                   : "0"(SYS_##name), "g"((long)(arg6)),                \
                   "g"((long)(arg5)), "g"((long)(arg4)),                \
                     "g"((long)(arg3)), "g"((long)(arg2)),              \
                     "g"((long)(arg1))                                  \
                     : "memory")

#endif  /* i386+ mode */

/* file management and I/O, declare specific system calls using the
   ugliness included from fbt_syscalls.h */
/* one possible implementation of fbt_suicide is an exit system call with an
   exit number, but this will not work if the stack is corrupted, so we just use
   a hlt instruction that causes a low level fault and terminates the
   program. sys_exit terminates the program and never returns, that's why we
   don't need to worry about saving the ebx register or return value. */
#define fbt_suicide(exitnr) __asm__ volatile("movl %0, %%eax\n"         \
                                             "movl %1, %%ebx\n"         \
                                             "int $0x80"                \
                                             : /* no return value */    \
                                             : "i"(SYS_exit),           \
                                               "i"((long)(exitnr))      \
                                             : "memory")
  
#define fbt_write(fd, buf, count, res) _syscall3(write, fd, buf, count, res)
#define fbt_open(pathname, flags, mode, res) _syscall3(open, pathname, flags, \
                                                       mode, res)
#define fbt_close(fd, res) _syscall1(close, fd, res)
#define fbt_lseek(fd, offset, whence, res) _syscall3(lseek, fd, offset, \
                                                     whence, res)
#define fbt_fstat(fd, stat, res) _syscall2(fstat, fd, stat, res)
#define fbt_mmap(addr, length, prot, flags, fd, offset, res) \
  _syscall6(mmap, addr, length, prot, flags, fd, offset, res)
#define fbt_munmap(addr, length, res) _syscall2(munmap, addr, length, res)
#define fbt_mprotect(addr, len, prot, res) _syscall3(mprotect, addr, len, \
                                                     prot, res)
#define fbt_signal(sig, handler, res) _syscall2(signal, sig, handler, res)
#define fbt_sigaction(sig, act, oldact, res) _syscall3(sigaction, sig, act, \
                                                       oldact, res)
#define fbt_clone(flags, stack, ptid, newtls, ctid, res) \
  _syscall5(clone, flags, stack, ptid, newtls, ctid, res)
#define fbt_rt_sigaction(sig, act, oldact, res) _syscall3(rt_sigaction, sig, \
                                                          act, oldact, res)

#endif  /* FBT_SYSCALLS_H */
