/**
 * Unit tests for the memory management unit
 *
 * Copyright (c) 2010 ETH Zurich
 *   Mathias Payer <mathias.payer@nebelwelt.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 */

#include "gtest/gtest.h"

#include "libfastbt.h"
#include "fbt_debug.h"
#include "fbt_mem_mgmt.h"

TEST(MemMgmtTest, InitTLS)
{
  DEBUG_START();
  struct thread_local_data *tld = fbt_init_tls();
  EXPECT_EQ(tld->trans.tld, tld);
  EXPECT_NE(tld->chunk, (void*)0);
  EXPECT_NE(tld->chunk->size, 0);
  EXPECT_NE(tld->stack, (void*)0);
  EXPECT_NE(tld->mappingtable, (void*)0);
  fbt_mem_free(tld);
  DEBUG_END();
}

TEST(MemMgmtTest, AllocateNewTrampolines)
{
  DEBUG_START();
  struct thread_local_data *tld = fbt_init_tls();
  int count = 0;
  struct trampoline *trampo = tld->trans.trampos;
  while (trampo != 0) {
    count++;
    trampo = trampo->next;
  }
  EXPECT_EQ(count, 0);
  fbt_allocate_new_trampolines(tld);
  /* reset and count again */
  count = 0;
  trampo = tld->trans.trampos;
  while (trampo != 0) {
    count++;
    trampo = trampo->next;
  }
  EXPECT_EQ(count, 1024); /* nr alloc trampos */
  fbt_mem_free(tld);  
  DEBUG_END();
}

TEST(MemMgmtTest, AllocateNewCodeCache)
{
  DEBUG_START();
  struct thread_local_data *tld = fbt_init_tls();
  void *transl_instr = tld->trans.transl_instr;
  EXPECT_NE(tld->trans.transl_instr, (void*)0);
  fbt_allocate_new_code_cache(tld);
  EXPECT_NE(tld->trans.transl_instr, (void*)0);
  EXPECT_NE(tld->trans.transl_instr, transl_instr);
  fbt_mem_free(tld);  
  DEBUG_END();
}

TEST(MemMgmtTest, StressTest)
{
  DEBUG_START();
  struct thread_local_data *tld = fbt_init_tls();
  /* allocates about 2GB memory */
  for (int i=0; i<128; ++i) {
    fbt_allocate_new_code_cache(tld);
    fbt_allocate_new_trampolines(tld);
  }
  fbt_mem_free(tld);
  DEBUG_END();
}
